from typing import Annotated
from uuid import UUID

from fastapi import APIRouter, Depends

from fastfood_two.application.contracts.requests import AddMenuDTO, UpdateMenuDTO
from fastfood_two.application.contracts.responses import MenuDTO
from fastfood_two.application.usecases.menu.add_menu import AddMenu
from fastfood_two.application.usecases.menu.get_all_menus import GetAllMenus
from fastfood_two.application.usecases.menu.update_menu import UpdateMenu
from fastfood_two.presentation.fastapi_backend.depends.stub import Stub
from fastfood_two.presentation.fastapi_backend.pdmodels.menu import (
    AddMenuPDModel,
    UpdateMenuPDModel,
)

router = APIRouter(prefix="/menu", tags=["Menu"])


@router.get("/", response_model=list[MenuDTO])
async def get_all_menus(
    usecase: Annotated[GetAllMenus, Depends(Stub(GetAllMenus))],
) -> list[MenuDTO]:
    """Get all menus.

    Endpoint returns list of all available food menus

    Parameters
    ----------
    None

    Returns
    -------
    list[MenuDTO]
        list of created menu items
    """
    menus = await usecase()
    return menus


@router.post("/", response_model=MenuDTO)
async def add_menu(
    request: AddMenuPDModel,
    usecase: Annotated[AddMenu, Depends(Stub(AddMenu))],
) -> MenuDTO:
    """Add menus.

    Endpoint allows to add new food menu

    Parameters
    ----------
    title: str
        title of the menu
    description: str
        description of the menu

    Returns
    -------
    MenuDTO
        created menu
    """
    menus = await usecase(request=AddMenuDTO(title=request.title, description=request.description))
    return menus


@router.patch("/{menu_id}", response_model=MenuDTO)
async def update_menu(
    menu_id: UUID,
    request: UpdateMenuPDModel,
    usecase: Annotated[UpdateMenu, Depends(Stub(UpdateMenu))],
) -> MenuDTO:
    """Update menus.

    Endpoint allows to update food menu

    Parameters
    ----------
    title: str | None
        new title of the menu or None if you don't want to update it
    description: str | None
        new description of the menu or None if you don't want to update it

    Returns
    -------
    MenuDTO
        updated menu
    """
    menus = await usecase(
        request=UpdateMenuDTO(
            id=menu_id,
            title=request.title,
            description=request.description,
        )
    )
    return menus
